/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Transport.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Transport_H
#define OpenSOAP_Transport_H

#include <OpenSOAP/ByteArray.h>
#include <OpenSOAP/Envelope.h>

/**
 * @file OpenSOAP/Transport.h
 * @brief OpenSOAP API Transport Processing
 * @author
 *    Tomonori Bando
 */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    /**
     * @typedef struct tagOpenSOAPTransport OpenSOAPTransport
     * @brief OpenSOAPTransport Structure Type Definition
     */
    typedef struct tagOpenSOAPTransport OpenSOAPTransport;

    /**
     * @typedef OpenSOAPTransport    *OpenSOAPTransportPtr
     * @brief OpenSOAPTransport Pointer Type Definition
     */
    typedef OpenSOAPTransport    *OpenSOAPTransportPtr;

    /**
      * @fn int OpenSOAPTransportCreate(OpenSOAPTransportPtr * t)
      * @brief Create Transport instance
      * @param
      *    t OpenSOAPTransportPtr * [out] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportCreate(OpenSOAPTransportPtr * /* [out] */ t);

    /**
      * @fn int OpenSOAPTransportRelease(OpenSOAPTransportPtr t)
      * @brief Release OpenSOAP Transport
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportRelease(OpenSOAPTransportPtr /* [in] */ t);

    /**
      * @fn int OpenSOAPTransportSend(OpenSOAPTransportPtr t, OpenSOAPEnvelopePtr soap_env)
      * @brief Send SOAP Envelope as ByteArray
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    soap_env OpenSOAPByteArrayPtr [in] ((|soap_env|)) SOAP Envelope as ByteArray
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSend(OpenSOAPTransportPtr /* [in] */ t,
                          OpenSOAPEnvelopePtr /* [in] */ soap_env);

    /**
      * @fn int OpenSOAPTransportReceive(OpenSOAPTransportPtr t, OpenSOAPEnvelopePtr * soap_env)
      * @brief Receive SOAP Message
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    soap_env OpenSOAPEnvelopePtr * [in, out] ((|soap_env|)) SOAP Envelope
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportReceive(OpenSOAPTransportPtr /* [in] */ t,
                             OpenSOAPEnvelopePtr * /* [in, out] */ soap_env);

    /**
      * @fn int OpenSOAPTransportConnect(OpenSOAPTransportPtr t)
      * @brief Connect to end point
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportConnect(OpenSOAPTransportPtr /* [in] */ t);

    /**
      * @fn int OpenSOAPTransportDisconnect(OpenSOAPTransportPtr t)
      * @brief Disconnect from end point
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportDisconnect(OpenSOAPTransportPtr /* [in] */ t);

    /**
      * @fn int OpenSOAPTransportInvokeByteArray(OpenSOAPTransportPtr t, OpenSOAPByteArrayPtr request, OpenSOAPByteArrayPtr response, int * tp_status)
      * @brief SOAP Service call using OpenSOAPByteArray
      * @param
      *    t OpenSOAPTransportPtr ((|t|)) OpenSOAP Transport pointer
      * @param
      *    request OpenSOAPEnvelopePtr [in] ((|request|)) SOAP request data
      * @param
      *    response OpenSOAPByteArrayPtr [out] ((|response|)) SOAP response data
      * @param
      *    tp_status int * [out] ((|tp_status|)) Transport status value
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportInvokeByteArray(OpenSOAPTransportPtr /* [in] */ t,
                                     OpenSOAPByteArrayPtr /* [in] */ request,
                                     OpenSOAPByteArrayPtr /* [out] */ response,
                                     int * /* [out] */ tp_status);
    
    /**
      * @fn int OpenSOAPTransportInvoke(OpenSOAPTransportPtr t, OpenSOAPEnvelopePtr request, OpenSOAPEnvelopePtr * response)
      * @brief SOAP call
      * @param
      *    t OpenSOAPTransportPtr [in] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    request OpenSOAPEnvelopePtr [in] ((|request|)) SOAP request Envelope
      * @param
      *    response OpenSOAPEnvelopePtr * [in, out] ((|response|)) SOAP response Envelope
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportInvoke(OpenSOAPTransportPtr /* [in] */ t,
                            OpenSOAPEnvelopePtr /* [in] */ request,
                            OpenSOAPEnvelopePtr * /* [in, out] */ response);
    
    /**
      * @fn int OpenSOAPTransportSetSOAPAction(OpenSOAPTransportPtr t, const char * soap_action)
      * @brief Set SOAP-Action Header (Some-URI)
      * @param
      *    t OpenSOAPTransportPtr [in, out] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    soap_action const char * [in] ((|soap_action|)) soap-action
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSetSOAPAction(OpenSOAPTransportPtr /* [in, out] */ t,
                                   const char * /* [in] */ soap_action);

    /**
      * @fn int OpenSOAPTransportSetURL(OpenSOAPTransportPtr t, const char * url)
      * @brief Set request URL
      * @param
      *    t OpenSOAPTransportPtr [in, out] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    url const char * [in] ((|url|)) URL
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSetURL(OpenSOAPTransportPtr /* [in, out] */ t,
                            const char * /* [in] */ url);

    /**
      * @fn int OpenSOAPTransportSetHeader(OpenSOAPTransportPtr t, const char * header_name, const char * header_value)
      * @brief Set Header
      * @param
      *    t OpenSOAPTransportPtr [in, out] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    header_name const char * [in] ((|header_name|)) header name
      * @param
      *    header_value const char * [in] ((|header_value|)) header's value
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSetHeader(OpenSOAPTransportPtr /* [in, out] */ t,
                               const char * /* [in] */ header_name,
                               const char * /* [in] */ header_value);

    /**
      * @fn int OpenSOAPTransportSetCharset(OpenSOAPTransportPtr t, const char * charset)
      * @brief Set Character Set
      * @param
      *    t OpenSOAPTransportPtr [in, out] ((|t|)) OpenSOAP Transport pointer
      * @param
      *    charset const char * [in] ((|charset|)) character set
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSetCharset(OpenSOAPTransportPtr /* [out] */ t,
                                const char * /* [in] */ charset);

    /**
      * @fn int OpenSOAPTransportSMTPSetHostname(OpenSOAPTransportPtr s)
      * @brief Set SMTP Host
      * @param
      *    s OpenSOAPTransportPtr [out] ((|s|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSMTPSetHostname(OpenSOAPTransportPtr /* [in, out] */ s);

    /**
      * @fn int OpenSOAPTransportSMTPSetFrom(OpenSOAPTransportPtr t)
      * @brief Set SMTP From Header
      * @param
      *    t OpenSOAPTransportPtr [out] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSMTPSetFrom(OpenSOAPTransportPtr /* [in, out] */ t);

    /**
      * @fn int OpenSOAPTransportSMTPSetTo(OpenSOAPTransportPtr t)
      * @brief Set SMTP To Header
      * @param
      *    t OpenSOAPTransportPtr [out] ((|t|)) OpenSOAP Transport pointer
      * @return
      *    Error Code
      */
    int
    OPENSOAP_API
    OpenSOAPTransportSMTPSetTo(OpenSOAPTransportPtr /* [in, out] */ t);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Transport_H */
